package numarea;

import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Scanner;

import javax.imageio.ImageIO;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileFilter;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;

public class FileIO {
	private	static	GridField	gridfield;
	public		static	String	pathname = null;
	public		static	int	savedPoint;
	public		static	int savedLevel;	
	
	private static FileFilter xmlfilter = new FileFilter(){
		@Override
		public boolean accept(File file) {
			if(file.isDirectory()) return true;
			return file.getName().endsWith("xml");
		}
		@Override
		public String getDescription() {
			return "xml file (*.xml)" ;
		}
	};
	
	private static FileFilter giffilter = new FileFilter(){
		@Override
		public boolean accept(File file) {
			if(file.isDirectory()) return true;
			return file.getName().endsWith("gif");
		}
		@Override
		public String getDescription() {
			return "gif file (*.gif)" ;
		}
	};
	
	private static		JFileChooser fileDialog = null;	
	
	// -------------------------------------------------------------------------
	//		XMLファイルのセーブ
	// -------------------------------------------------------------------------saveFile 
	public static void saveFile(GridField grid,ControlDialog fcdialog) {
		gridfield = grid;
		if(fileDialog == null){
			fileDialog = new JFileChooser(new File("."));
			fileDialog.addChoosableFileFilter(xmlfilter);
		}
		if (fileDialog.showSaveDialog(gridfield) == JFileChooser.CANCEL_OPTION) return ;
		String filename = fileDialog.getSelectedFile().toString();
		if(!filename.endsWith(".xml")) filename = filename + ".xml";
		File file = new File (filename);
		if(file.exists()){
			int response = JOptionPane.showConfirmDialog(null,
                    "既に同じ名前のファイルが存在します。上書きしますか？", "警告", JOptionPane.YES_NO_OPTION);
			
			if(response != JOptionPane.OK_OPTION)
				return ;
		}
		
		saveFile( grid, file );
		pathname = filename;
	}
	
	public static void saveFile(GridField grid, File file ) {
		PrintWriter writer = null;
		
	    xml.Problem xmlproblem = new xml.Problem();		
	    xmlproblem.setWidth("" + (gridfield.getBoardWidth()));	
	    xmlproblem.setHeight("" + (gridfield.getBoardHeight()));
	    xmlproblem.setName("NumberArea");
	    xmlproblem.setDifficult(""+gridfield.getDifficultypoint());
	    xmlproblem.setPattern(gridfield.currentPatternString());
	    xmlproblem.setPattern2(gridfield.currentPattern2String());
	    xmlproblem.setPattern3(gridfield.currentPattern3String());
	    xmlproblem.setHint(gridfield.currentAreaString());
	    
		try{
			writer = new PrintWriter(file);
			writer.write("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\n");
			xmlproblem.makeTextElement(writer);
		}catch(FileNotFoundException e){
			return ;
		}finally{
			if(writer != null)
				writer.close();
		}		
	}
	
	// -------------------------------------------------------------------------
	//		XMLファイルのロード
	// -------------------------------------------------------------------------	
	public static	File getloadFile(GridField grid,ControlDialog fcdialog){	
		gridfield = grid;
		if(fileDialog == null){
			fileDialog = new JFileChooser(new File("."));
			fileDialog.addChoosableFileFilter(xmlfilter);
		}
		if (fileDialog.showOpenDialog(gridfield) == JFileChooser.CANCEL_OPTION) return null;
		File file = fileDialog.getSelectedFile();
		
		return	file;
	}
		
	public static xml.Problem openXMLFile(File file) {
		try {
		    DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
		    DocumentBuilder builder = dbfactory.newDocumentBuilder();
		    Document doc = builder.parse(file);
		    xml.Problem xmlproblem = new xml.Problem(doc);
		    return	xmlproblem;

		}catch(Exception e) {
			e.printStackTrace();
		}
		return	null;
	}
	
	public static	void loadFile(File file)	 {
			
		pathname = file.toString();
		try{
		    DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
		    DocumentBuilder builder = dbfactory.newDocumentBuilder();
		    Document doc = builder.parse(file);
		    xml.Problem xmlproblem = new xml.Problem(doc);
			
			int w = Integer.parseInt(xmlproblem.getWidth());
			int h = Integer.parseInt(xmlproblem.getHeight());
			String pstr = xmlproblem.getDifficult();
			savedPoint = pstr!=null ? Integer.parseInt(pstr) : -1;
			String lstr = xmlproblem.getLevel();
			savedLevel = lstr!=null ? Integer.parseInt(lstr) : -1;
			if( ! gridfield.setGridSize(w, h) )	// サイズ不可能
				return;
			
			// load pattern
			boolean [][] pattern = new boolean[w][h];
			String patternstr = xmlproblem.getPattern();
			if( patternstr!= null && patternstr.length()>0 ) {
				Scanner scan = new Scanner(patternstr);
				for(int j=0;j<h;j++){
					for(int i=0;i<w;i++){
						String s = scan.next();
						pattern[i][j] = s.equals("P");
					}
				}
			}
			gridfield.setPattern(pattern);
			
			// load pattern2
			boolean [][] pattern2 = new boolean[w][h];
			String pattern2str = xmlproblem.getPattern2();
			if( pattern2str!= null && pattern2str.length()>0 ) {
				Scanner scan = new Scanner(pattern2str);
				for(int j=0;j<h;j++){
					for(int i=0;i<w;i++){
						String s = scan.next();
						pattern2[i][j] = s.equals("Q");
					}
				}
			}
			gridfield.setPattern2(pattern2);
			
			// load pattern3
			int [][] pattern3 = new int[w][h];
			String pattern3str = xmlproblem.getPattern3();
			if( pattern3str!= null && pattern3str.length()>0 ) {
				Scanner scan = new Scanner(pattern3str);
				for(int j=0;j<h;j++){
					for(int i=0;i<w;i++){
						String s = scan.next();
						pattern3[i][j] = Integer.valueOf(s);
					}
				}
			}
			gridfield.setPreNumTable(pattern3);
			
			// load pattern
			int [][] area = new int[w][h];
			String areastr = xmlproblem.getHint();
			if( areastr!= null && areastr.length()>0 ) {
				Scanner scan = new Scanner(areastr);
				for(int j=0;j<h;j++){
					for(int i=0;i<w;i++){
						String s = scan.next();
						area[i][j] = Integer.parseInt(s);
					}
				}
			}
			gridfield.setArea(area);	
			
			gridfield.refresh();
			
			if( gridfield.designpanel != null )
				gridfield.designpanel.setInputSize(w,h);
		}catch(Exception e) {}		
	}
	
	// -------------------------------------------------------------------------
	//		GIFファイルのロード
	// -------------------------------------------------------------------------	
	public static	File getloadGifFile(GridField grid,ControlDialog fcdialog){	
		gridfield = grid;
		if(fileDialog == null){
			fileDialog = new JFileChooser(new File("."));
			fileDialog.addChoosableFileFilter(giffilter);
		}
		if (fileDialog.showOpenDialog(gridfield) == JFileChooser.CANCEL_OPTION)
			return null;
		File file = fileDialog.getSelectedFile();
		
		return	file;
	}
	
	public static void loadGifFile( File file ) {
 		BufferedImage img = null;
		try {
			img = ImageIO.read(file);
		} catch (IOException e) {
			e.printStackTrace();
		}
		if( img == null )
			return;
		
 		int w = img.getWidth();
 		int h = img.getHeight();
		if( ! gridfield.setGridSize(w,h) )			// サイズ不可能
			return;
		gridfield.refresh();

 		for( int y=0; y<h; ++y ) {
 			for( int x=0; x<w; ++x ) {
 				int	rgb = img.getRGB(x, y);
 				int	av = (rgb & 0xff)+((rgb>>8)&0xff)+((rgb>>16)&0xff) / 3;
 				if( av< 0x80 )
 					gridfield.changeCell(x,y);
 			}
 		}
		gridfield.refresh();
	}
}

