package numarea;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Graphics;
import java.util.ArrayList;

import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import numarea.GA.GeneticPoolInfo;

/* ================================================================
 *  グラフ　パネル
 */
public class GraphPanel extends JPanel implements ChangeListener {
	private	GridField	gridfield;
	private	Sheet		canvas;
	
	JCheckBox	logCheck 		= new JCheckBox("Log");
	JCheckBox	mutateCheck	= new JCheckBox("Mutate Count");
	
	GraphPanel(GridField grid) {
		this.gridfield = grid;
		grid.graphpanel = this;	
		
		this.setLayout(new BorderLayout());
				
		JPanel npanel = new JPanel();
		npanel.add(logCheck);
		logCheck.addChangeListener(this);
		
		npanel.add(mutateCheck);
		mutateCheck.addChangeListener(this);
		
		add(npanel,BorderLayout.NORTH);
		
		canvas = new Sheet();
		add(canvas,BorderLayout.CENTER);
	}
	
	public void stateChanged(ChangeEvent e) {
		canvas.repaint();
	}
	
	public	void	update() {
		canvas.repaint();
	}
	
	class	Sheet	extends JPanel {
		boolean	bylog;
		boolean	mclog;
		int	w;
		int	h;
		int	inheight;
		int	inwidth;
		int	vgap = 30;
		int	hgap = 30;
		int	len;
		int	max;
		
		Sheet() {
			setBackground(Color.WHITE);
		}
		
		public void paint(Graphics g) {
			w = getWidth();
			h = getHeight();
			g.setColor(Color.white);
			g.fillRect(0,0,w,h);
			
			g.setColor(Color.black);
			drawGraph(g);
		}
		
		private	void	drawGraph(Graphics g) {
			bylog = logCheck.isSelected();
			mclog = mutateCheck.isSelected();
				
			if( gridfield.ga == null )
				return;
			
			ArrayList<GeneticPoolInfo>	gilist = gridfield.ga.geneinfo;
			if( gilist == null || gilist.size()<2 )
				return;
			max = gilist.get(0).selectmax;
			len = gilist.size();
			
			inheight	= h - 2*vgap;
			inwidth	= w - 2*hgap;
			
			drawAxis(g);

			for( int i=1; i<len; ++i ) {
				g.setColor(Color.black);
				
				int	pv, cv, py, cy, px, cx;
				pv = gilist.get(i-1).selectmax;
				cv = gilist.get(i).selectmax;
				py = - grheight(pv) + h - vgap;
				cy = - grheight(cv) + h - vgap;
				px = grtime(i) + hgap;
				cx = grtime(i+1)   + hgap;			
				g.drawLine( px, py, cx, cy );
				
				pv = gilist.get(i-1).selectmin;
				cv = gilist.get(i).selectmin;
				py = - grheight(pv) + h - vgap;
				cy = - grheight(cv) + h - vgap;				
				px = grtime(i) + hgap;
				cx = grtime(i+1)   + hgap;			
				g.drawLine( px, py, cx, cy );
				
				g.setColor(Color.red);
				
				double	pa, ca;
				pa = gilist.get(i-1).average;
				ca = gilist.get(i).average;
				py = - grheight(pa) + h - vgap;
				cy = - grheight(ca) + h - vgap;				
				px = grtime(i) + hgap;
				cx = grtime(i+1)   + hgap;			
				g.drawLine( px, py, cx, cy );	
				
				if( mclog ) {
					g.setColor(Color.blue);
					pv = gilist.get(i-1).mutatecount;
					cv = gilist.get(i).mutatecount;
					py = - grheight(pv) + h - vgap;
					cy = - grheight(cv) + h - vgap;				
					px = grtime(i) + hgap;
					cx = grtime(i+1)   + hgap;			
					g.drawLine( px, py, cx, cy );	
				}
			}
			
		}
		
		private	void	drawAxis(Graphics g) {
			g.drawLine( hgap/2, h-vgap, w-hgap/2, h-vgap );
			g.drawLine( hgap, vgap/2, hgap, h-vgap/2 );
			g.drawLine( w-hgap, vgap/2, w-hgap, h-vgap/2 );

			// hticks
			int	hresolution[] = {1,5,10,50,100,500,1000};
			int	hticklength = 3;
			for( int i=0; i<hresolution.length; ++i, hticklength+=3 ) {
				int r = hresolution[i];
				if( len/r > 50 )
					continue;
				for( int t=r; t<=len; t+=r ) {
					int x = grtime(t) + hgap;
					g.drawLine( x, h - vgap, x, h - vgap+hticklength );
				}
			}
			
			// vticks
			if( bylog ) {	// 対数メモリ

				for( int j=1; j<=max; j*=10 ) {
					for( int i=1; i<10; ++i ) {
						int	v = i*j;
						int	y = - grheight(v) + h - vgap;
						if( y < vgap/2 )
							continue;
						
						int	ticklen = (i==1||i==5) ? 10 : 5;
						g.drawLine(hgap  ,y,hgap-ticklen,  y);
						g.drawLine(w-hgap,y,w-hgap+ticklen,y);
					}
				}
				
			} else {
				
			}
		}
		
		private	int		grheight( double val ) {
			if( bylog ) {
				return (int)(Math.log(val+1) * inheight / Math.log(max+1));
			} else {
				return	(int) (val * inheight / max);
			}
		}
		
		private	int		grtime( int t ) {
			return	t*inwidth/(len+1);
		}
	}
}
