/*
 * The Relaxer artifact
 * Copyright (c) 2000-2003, ASAMI Tomoharu, All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * - Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer. 
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */
package xml;

import java.io.*;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Writer;
import java.net.URL;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

/**
 * <b>Problem</b> is generated from numarea.rng by Relaxer.
 * This class is derived from:
 * 
 * <!-- for programmer
 * <element name="problem">
 *   <attribute name="width"/>
 *   <attribute name="height"/>
 *   <optional>
 *     <attribute name="name"/>
 *   </optional>
 *   <optional>
 *     <attribute name="author"/>
 *   </optional>
 *   <optional>
 *     <attribute name="level"/>
 *   </optional>
 *   <optional>
 *     <attribute name="difficult"/>
 *   </optional>
 *   <optional>
 *     <ref name="Control"/>
 *   </optional>
 *   <optional>
 *     <ref name="Pattern"/>
 *   </optional>
 *   <optional>
 *     <ref name="Pattern2"/>
 *   </optional>
 *   <optional>
 *     <ref name="Pattern3"/>
 *   </optional>
 *   <optional>
 *     <ref name="Hint"/>
 *   </optional>
 *   <optional>
 *     <ref name="Answer"/>
 *   </optional>
 *   <optional>
 *     <ref name="Comment"/>
 *   </optional>
 * </element>
 * -->
 * <!-- for javadoc -->
 * <pre> &lt;element name="problem"&gt;
 *   &lt;attribute name="width"/&gt;
 *   &lt;attribute name="height"/&gt;
 *   &lt;optional&gt;
 *     &lt;attribute name="name"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;attribute name="author"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;attribute name="level"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;attribute name="difficult"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Control"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Pattern"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Pattern2"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Pattern3"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Hint"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Answer"/&gt;
 *   &lt;/optional&gt;
 *   &lt;optional&gt;
 *     &lt;ref name="Comment"/&gt;
 *   &lt;/optional&gt;
 * &lt;/element&gt;
 * </pre>
 *
 * @version numarea.rng (Mon Apr 07 16:01:22 JST 2014)
 * @author  Relaxer 1.0 (http://www.relaxer.org)
 */
public class Problem implements java.io.Serializable, Cloneable {
    private String width_;
    private String height_;
    private String name_;
    private String author_;
    private String level_;
    private String difficult_;
    private String control_;
    private String pattern_;
    private String pattern2_;
    private String pattern3_;
    private String hint_;
    private String answer_;
    private String comment_;

    /**
     * Creates a <code>Problem</code>.
     *
     */
    public Problem() {
        width_ = "";
        height_ = "";
    }

    /**
     * Creates a <code>Problem</code>.
     *
     * @param source
     */
    public Problem(Problem source) {
        setup(source);
    }

    /**
     * Creates a <code>Problem</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public Problem(RStack stack) {
        setup(stack);
    }

    /**
     * Creates a <code>Problem</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public Problem(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Creates a <code>Problem</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public Problem(Element element) {
        setup(element);
    }

    /**
     * Creates a <code>Problem</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file);
    }

    /**
     * Creates a <code>Problem</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(uri);
    }

    /**
     * Creates a <code>Problem</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(url);
    }

    /**
     * Creates a <code>Problem</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(in);
    }

    /**
     * Creates a <code>Problem</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(is);
    }

    /**
     * Creates a <code>Problem</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public Problem(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(reader);
    }

    /**
     * Initializes the <code>Problem</code> by the Problem <code>source</code>.
     *
     * @param source
     */
    public void setup(Problem source) {
        int size;
        setWidth(source.getWidth());
        setHeight(source.getHeight());
        setName(source.getName());
        setAuthor(source.getAuthor());
        setLevel(source.getLevel());
        setDifficult(source.getDifficult());
        setControl(source.getControl());
        setPattern(source.getPattern());
        setPattern2(source.getPattern2());
        setPattern3(source.getPattern3());
        setHint(source.getHint());
        setAnswer(source.getAnswer());
        setComment(source.getComment());
    }

    /**
     * Initializes the <code>Problem</code> by the Document <code>doc</code>.
     *
     * @param doc
     */
    public void setup(Document doc) {
        setup(doc.getDocumentElement());
    }

    /**
     * Initializes the <code>Problem</code> by the Element <code>element</code>.
     *
     * @param element
     */
    public void setup(Element element) {
        init(element);
    }

    /**
     * Initializes the <code>Problem</code> by the Stack <code>stack</code>
     * that contains Elements.
     * This constructor is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     */
    public void setup(RStack stack) {
        init(stack.popElement());
    }

    /**
     * @param element
     */
    private void init(Element element) {
        RStack stack = new RStack(element);
        width_ = URelaxer.getAttributePropertyAsString(element, "width");
        height_ = URelaxer.getAttributePropertyAsString(element, "height");
        name_ = URelaxer.getAttributePropertyAsString(element, "name");
        author_ = URelaxer.getAttributePropertyAsString(element, "author");
        level_ = URelaxer.getAttributePropertyAsString(element, "level");
        difficult_ = URelaxer.getAttributePropertyAsString(element, "difficult");
        control_ = URelaxer.getElementPropertyAsStringByStack(stack, "control");
        pattern_ = URelaxer.getElementPropertyAsStringByStack(stack, "pattern");
        pattern2_ = URelaxer.getElementPropertyAsStringByStack(stack, "pattern2");
        pattern3_ = URelaxer.getElementPropertyAsStringByStack(stack, "pattern3");
        hint_ = URelaxer.getElementPropertyAsStringByStack(stack, "hint");
        answer_ = URelaxer.getElementPropertyAsStringByStack(stack, "answer");
        comment_ = URelaxer.getElementPropertyAsStringByStack(stack, "comment");
    }

    /**
     * @return Object
     */
    public Object clone() {
        return (new Problem(this));
    }

    /**
     * Creates a DOM representation of the object.
     * Result is appended to the Node <code>parent</code>.
     *
     * @param parent
     */
    public void makeElement(Node parent) {
        Document doc;
        if (parent instanceof Document) {
            doc = (Document)parent;
        } else {
            doc = parent.getOwnerDocument();
        }
        Element element = doc.createElement("problem");
        int size;
        if (this.width_ != null) {
            URelaxer.setAttributePropertyByString(element, "width", this.width_);
        }
        if (this.height_ != null) {
            URelaxer.setAttributePropertyByString(element, "height", this.height_);
        }
        if (this.name_ != null) {
            URelaxer.setAttributePropertyByString(element, "name", this.name_);
        }
        if (this.author_ != null) {
            URelaxer.setAttributePropertyByString(element, "author", this.author_);
        }
        if (this.level_ != null) {
            URelaxer.setAttributePropertyByString(element, "level", this.level_);
        }
        if (this.difficult_ != null) {
            URelaxer.setAttributePropertyByString(element, "difficult", this.difficult_);
        }
        if (this.control_ != null) {
            URelaxer.setElementPropertyByString(element, "control", this.control_);
        }
        if (this.pattern_ != null) {
            URelaxer.setElementPropertyByString(element, "pattern", this.pattern_);
        }
        if (this.pattern2_ != null) {
            URelaxer.setElementPropertyByString(element, "pattern2", this.pattern2_);
        }
        if (this.pattern3_ != null) {
            URelaxer.setElementPropertyByString(element, "pattern3", this.pattern3_);
        }
        if (this.hint_ != null) {
            URelaxer.setElementPropertyByString(element, "hint", this.hint_);
        }
        if (this.answer_ != null) {
            URelaxer.setElementPropertyByString(element, "answer", this.answer_);
        }
        if (this.comment_ != null) {
            URelaxer.setElementPropertyByString(element, "comment", this.comment_);
        }
        parent.appendChild(element);
    }

    /**
     * Initializes the <code>Problem</code> by the File <code>file</code>.
     *
     * @param file
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(File file) throws IOException, SAXException, ParserConfigurationException {
        setup(file.toURL());
    }

    /**
     * Initializes the <code>Problem</code>
     * by the String representation of URI <code>uri</code>.
     *
     * @param uri
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(String uri) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(uri, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Problem</code> by the URL <code>url</code>.
     *
     * @param url
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(URL url) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(url, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Problem</code> by the InputStream <code>in</code>.
     *
     * @param in
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputStream in) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(in, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Problem</code> by the InputSource <code>is</code>.
     *
     * @param is
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(InputSource is) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(is, UJAXP.FLAG_NONE));
    }

    /**
     * Initializes the <code>Problem</code> by the Reader <code>reader</code>.
     *
     * @param reader
     * @exception IOException
     * @exception SAXException
     * @exception ParserConfigurationException
     */
    public void setup(Reader reader) throws IOException, SAXException, ParserConfigurationException {
        setup(UJAXP.getDocument(reader, UJAXP.FLAG_NONE));
    }

    /**
     * Creates a DOM document representation of the object.
     *
     * @exception ParserConfigurationException
     * @return Document
     */
    public Document makeDocument() throws ParserConfigurationException {
        Document doc = UJAXP.makeDocument();
        makeElement(doc);
        return (doc);
    }

    /**
     * Gets the String property <b>width</b>.
     *
     * @return String
     */
    public final String getWidth() {
        return (width_);
    }

    /**
     * Sets the String property <b>width</b>.
     *
     * @param width
     */
    public final void setWidth(String width) {
        this.width_ = width;
    }

    /**
     * Gets the String property <b>height</b>.
     *
     * @return String
     */
    public final String getHeight() {
        return (height_);
    }

    /**
     * Sets the String property <b>height</b>.
     *
     * @param height
     */
    public final void setHeight(String height) {
        this.height_ = height;
    }

    /**
     * Gets the String property <b>name</b>.
     *
     * @return String
     */
    public final String getName() {
        return (name_);
    }

    /**
     * Sets the String property <b>name</b>.
     *
     * @param name
     */
    public final void setName(String name) {
        this.name_ = name;
    }

    /**
     * Gets the String property <b>author</b>.
     *
     * @return String
     */
    public final String getAuthor() {
        return (author_);
    }

    /**
     * Sets the String property <b>author</b>.
     *
     * @param author
     */
    public final void setAuthor(String author) {
        this.author_ = author;
    }

    /**
     * Gets the String property <b>level</b>.
     *
     * @return String
     */
    public final String getLevel() {
        return (level_);
    }

    /**
     * Sets the String property <b>level</b>.
     *
     * @param level
     */
    public final void setLevel(String level) {
        this.level_ = level;
    }

    /**
     * Gets the String property <b>difficult</b>.
     *
     * @return String
     */
    public final String getDifficult() {
        return (difficult_);
    }

    /**
     * Sets the String property <b>difficult</b>.
     *
     * @param difficult
     */
    public final void setDifficult(String difficult) {
        this.difficult_ = difficult;
    }

    /**
     * Gets the String property <b>control</b>.
     *
     * @return String
     */
    public final String getControl() {
        return (control_);
    }

    /**
     * Sets the String property <b>control</b>.
     *
     * @param control
     */
    public final void setControl(String control) {
        this.control_ = control;
    }

    /**
     * Gets the String property <b>pattern</b>.
     *
     * @return String
     */
    public final String getPattern() {
        return (pattern_);
    }

    /**
     * Sets the String property <b>pattern</b>.
     *
     * @param pattern
     */
    public final void setPattern(String pattern) {
        this.pattern_ = pattern;
    }

    /**
     * Gets the String property <b>pattern2</b>.
     *
     * @return String
     */
    public final String getPattern2() {
        return (pattern2_);
    }

    /**
     * Sets the String property <b>pattern2</b>.
     *
     * @param pattern2
     */
    public final void setPattern2(String pattern2) {
        this.pattern2_ = pattern2;
    }

    /**
     * Gets the String property <b>pattern3</b>.
     *
     * @return String
     */
    public final String getPattern3() {
        return (pattern3_);
    }

    /**
     * Sets the String property <b>pattern3</b>.
     *
     * @param pattern3
     */
    public final void setPattern3(String pattern3) {
        this.pattern3_ = pattern3;
    }

    /**
     * Gets the String property <b>hint</b>.
     *
     * @return String
     */
    public final String getHint() {
        return (hint_);
    }

    /**
     * Sets the String property <b>hint</b>.
     *
     * @param hint
     */
    public final void setHint(String hint) {
        this.hint_ = hint;
    }

    /**
     * Gets the String property <b>answer</b>.
     *
     * @return String
     */
    public final String getAnswer() {
        return (answer_);
    }

    /**
     * Sets the String property <b>answer</b>.
     *
     * @param answer
     */
    public final void setAnswer(String answer) {
        this.answer_ = answer;
    }

    /**
     * Gets the String property <b>comment</b>.
     *
     * @return String
     */
    public final String getComment() {
        return (comment_);
    }

    /**
     * Sets the String property <b>comment</b>.
     *
     * @param comment
     */
    public final void setComment(String comment) {
        this.comment_ = comment;
    }

    /**
     * Makes an XML text representation.
     *
     * @return String
     */
    public String makeTextDocument() {
        StringBuffer buffer = new StringBuffer();
        makeTextElement(buffer);
        return (new String(buffer));
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(StringBuffer buffer) {
        int size;
        buffer.append("<problem");
        if (width_ != null) {
            buffer.append(" width=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getWidth())));
            buffer.append("\"");
        }
        if (height_ != null) {
            buffer.append(" height=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getHeight())));
            buffer.append("\"");
        }
        if (name_ != null) {
            buffer.append(" name=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getName())));
            buffer.append("\"");
        }
        if (author_ != null) {
            buffer.append(" author=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getAuthor())));
            buffer.append("\"");
        }
        if (level_ != null) {
            buffer.append(" level=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getLevel())));
            buffer.append("\"");
        }
        if (difficult_ != null) {
            buffer.append(" difficult=\"");
            buffer.append(URelaxer.escapeAttrQuot(URelaxer.getString(getDifficult())));
            buffer.append("\"");
        }
        buffer.append(">");
        if (control_ != null) {
            buffer.append("<control>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getControl())));
            buffer.append("</control>");
        }
        if (pattern_ != null) {
            buffer.append("<pattern>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getPattern())));
            buffer.append("</pattern>");
        }
        if (pattern2_ != null) {
            buffer.append("<pattern2>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getPattern2())));
            buffer.append("</pattern2>");
        }
        if (pattern3_ != null) {
            buffer.append("<pattern3>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getPattern3())));
            buffer.append("</pattern3>");
        }
        if (hint_ != null) {
            buffer.append("<hint>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getHint())));
            buffer.append("</hint>");
        }
        if (answer_ != null) {
            buffer.append("<answer>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getAnswer())));
            buffer.append("</answer>");
        }
        if (comment_ != null) {
            buffer.append("<comment>");
            buffer.append(URelaxer.escapeCharData(URelaxer.getString(getComment())));
            buffer.append("</comment>");
        }
        buffer.append("</problem>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextElement(Writer buffer) throws IOException {
        int size;
        buffer.write("<problem");
        if (width_ != null) {
            buffer.write(" width=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getWidth())));
            buffer.write("\"");
        }
        if (height_ != null) {
            buffer.write(" height=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getHeight())));
            buffer.write("\"");
        }
        if (name_ != null) {
            buffer.write(" name=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getName())));
            buffer.write("\"");
        }
        if (author_ != null) {
            buffer.write(" author=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getAuthor())));
            buffer.write("\"");
        }
        if (level_ != null) {
            buffer.write(" level=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getLevel())));
            buffer.write("\"");
        }
        if (difficult_ != null) {
            buffer.write(" difficult=\"");
            buffer.write(URelaxer.escapeAttrQuot(URelaxer.getString(getDifficult())));
            buffer.write("\"");
        }
        buffer.write(">");
        if (control_ != null) {
            buffer.write("<control>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getControl())));
            buffer.write("</control>");
        }
        if (pattern_ != null) {
            buffer.write("<pattern>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getPattern())));
            buffer.write("</pattern>");
        }
        if (pattern2_ != null) {
            buffer.write("<pattern2>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getPattern2())));
            buffer.write("</pattern2>");
        }
        if (pattern3_ != null) {
            buffer.write("<pattern3>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getPattern3())));
            buffer.write("</pattern3>");
        }
        if (hint_ != null) {
            buffer.write("<hint>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getHint())));
            buffer.write("</hint>");
        }
        if (answer_ != null) {
            buffer.write("<answer>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getAnswer())));
            buffer.write("</answer>");
        }
        if (comment_ != null) {
            buffer.write("<comment>");
            buffer.write(URelaxer.escapeCharData(URelaxer.getString(getComment())));
            buffer.write("</comment>");
        }
        buffer.write("</problem>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextElement(PrintWriter buffer) {
        int size;
        buffer.print("<problem");
        if (width_ != null) {
            buffer.print(" width=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getWidth())));
            buffer.print("\"");
        }
        if (height_ != null) {
            buffer.print(" height=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getHeight())));
            buffer.print("\"");
        }
        if (name_ != null) {
            buffer.print(" name=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getName())));
            buffer.print("\"");
        }
        if (author_ != null) {
            buffer.print(" author=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getAuthor())));
            buffer.print("\"");
        }
        if (level_ != null) {
            buffer.print(" level=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getLevel())));
            buffer.print("\"");
        }
        if (difficult_ != null) {
            buffer.print(" difficult=\"");
            buffer.print(URelaxer.escapeAttrQuot(URelaxer.getString(getDifficult())));
            buffer.print("\"");
        }
        buffer.print(">");
        if (control_ != null) {
            buffer.print("<control>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getControl())));
            buffer.print("</control>");
        }
        if (pattern_ != null) {
            buffer.print("<pattern>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getPattern())));
            buffer.print("</pattern>");
        }
        if (pattern2_ != null) {
            buffer.print("<pattern2>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getPattern2())));
            buffer.print("</pattern2>");
        }
        if (pattern3_ != null) {
            buffer.print("<pattern3>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getPattern3())));
            buffer.print("</pattern3>");
        }
        if (hint_ != null) {
            buffer.print("<hint>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getHint())));
            buffer.print("</hint>");
        }
        if (answer_ != null) {
            buffer.print("<answer>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getAnswer())));
            buffer.print("</answer>");
        }
        if (comment_ != null) {
            buffer.print("<comment>");
            buffer.print(URelaxer.escapeCharData(URelaxer.getString(getComment())));
            buffer.print("</comment>");
        }
        buffer.print("</problem>");
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(StringBuffer buffer) {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     * @exception IOException
     */
    public void makeTextAttribute(Writer buffer) throws IOException {
    }

    /**
     * Makes an XML text representation.
     *
     * @param buffer
     */
    public void makeTextAttribute(PrintWriter buffer) {
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getWidthAsString() {
        return (URelaxer.getString(getWidth()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getHeightAsString() {
        return (URelaxer.getString(getHeight()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getNameAsString() {
        return (URelaxer.getString(getName()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAuthorAsString() {
        return (URelaxer.getString(getAuthor()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getLevelAsString() {
        return (URelaxer.getString(getLevel()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getDifficultAsString() {
        return (URelaxer.getString(getDifficult()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getControlAsString() {
        return (URelaxer.getString(getControl()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getPatternAsString() {
        return (URelaxer.getString(getPattern()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getPattern2AsString() {
        return (URelaxer.getString(getPattern2()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getPattern3AsString() {
        return (URelaxer.getString(getPattern3()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getHintAsString() {
        return (URelaxer.getString(getHint()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getAnswerAsString() {
        return (URelaxer.getString(getAnswer()));
    }

    /**
     * Gets the property value as String.
     *
     * @return String
     */
    public String getCommentAsString() {
        return (URelaxer.getString(getComment()));
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setWidthByString(String string) {
        setWidth(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setHeightByString(String string) {
        setHeight(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setNameByString(String string) {
        setName(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAuthorByString(String string) {
        setAuthor(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setLevelByString(String string) {
        setLevel(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setDifficultByString(String string) {
        setDifficult(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setControlByString(String string) {
        setControl(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setPatternByString(String string) {
        setPattern(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setPattern2ByString(String string) {
        setPattern2(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setPattern3ByString(String string) {
        setPattern3(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setHintByString(String string) {
        setHint(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setAnswerByString(String string) {
        setAnswer(string);
    }

    /**
     * Sets the property value by String.
     *
     * @param string
     */
    public void setCommentByString(String string) {
        setComment(string);
    }

    /**
     * Returns a String representation of this object.
     * While this method informs as XML format representaion, 
     *  it's purpose is just information, not making 
     * a rigid XML documentation.
     *
     * @return String
     */
    public String toString() {
        try {
            return (makeTextDocument());
        } catch (Exception e) {
            return (super.toString());
        }
    }

    /**
     * Tests if a Element <code>element</code> is valid
     * for the <code>Problem</code>.
     *
     * @param element
     * @return boolean
     */
    public static boolean isMatch(Element element) {
        if (!URelaxer.isTargetElement(element, "problem")) {
            return (false);
        }
        RStack target = new RStack(element);
        boolean $match$ = false;
        Element child;
        if (!URelaxer.hasAttributeHungry(target, "width")) {
            return (false);
        }
        $match$ = true;
        if (!URelaxer.hasAttributeHungry(target, "height")) {
            return (false);
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "control")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "pattern")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "pattern2")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "pattern3")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "hint")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "answer")) {
                target.popElement();
            }
        }
        $match$ = true;
        child = target.peekElement();
        if (child != null) {
            if (URelaxer.isTargetElement(child, "comment")) {
                target.popElement();
            }
        }
        $match$ = true;
        if (!target.isEmptyElement()) {
            return (false);
        }
        return (true);
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Problem</code>.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatch(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        return (isMatch(element));
    }

    /**
     * Tests if elements contained in a Stack <code>stack</code>
     * is valid for the <code>Problem</code>.
     * This method consumes the stack contents during matching operation.
     * This mehtod is supposed to be used internally
     * by the Relaxer system.
     *
     * @param stack
     * @return boolean
     */
    public static boolean isMatchHungry(RStack stack) {
        Element element = stack.peekElement();
        if (element == null) {
            return (false);
        }
        if (isMatch(element)) {
            stack.popElement();
            return (true);
        } else {
            return (false);
        }
    }
}
